@@ -33,6 +33,26 @@ typedef struct {
 
 PyTypeObject Proxy_Type;
 
+
+/* ------------------------------------------------------------------------- */
+
+static PyObject *identity_ref = NULL;
+static PyObject *
+identity(PyObject *self, PyObject *value)
+{
+    Py_INCREF(value);
+    return value;
+}
+
+/* ------------------------------------------------------------------------- */
+
+PyDoc_STRVAR(identity_doc, "Indentity function: returns the single argument.");
+
+static struct PyMethodDef module_functions[] = {
+    {"identity", identity, METH_O, identity_doc},
+    {NULL,       NULL}
+};
+
 /* ------------------------------------------------------------------------- */
 
 static PyObject *Proxy__ensure_wrapped(ProxyObject *self)
@@ -45,7 +65,6 @@ static PyObject *Proxy__ensure_wrapped(ProxyObject *self)
         if (self->factory) {
             wrapped = PyObject_CallFunctionObjArgs(self->factory, NULL);
             if (wrapped) {
-                Py_INCREF(wrapped);
                 self->wrapped = wrapped;
                 return wrapped;
             } else {
@@ -55,8 +74,6 @@ static PyObject *Proxy__ensure_wrapped(ProxyObject *self)
             PyErr_SetString(PyExc_ValueError, "Proxy hasn't been initiated: __factory__ is missing.");
             return NULL;
         }
-
-
     }
 }
 
@@ -116,7 +133,7 @@ static int Proxy_traverse(ProxyObject *self,
 {
     Py_VISIT(self->dict);
     Py_VISIT(self->wrapped);
-
+    Py_VISIT(self->factory);
     return 0;
 }
 
@@ -126,7 +143,7 @@ static int Proxy_clear(ProxyObject *self)
 {
     Py_CLEAR(self->dict);
     Py_CLEAR(self->wrapped);
-
+    Py_CLEAR(self->factory);
     return 0;
 }
 
@@ -145,17 +162,44 @@ static void Proxy_dealloc(ProxyObject *self)
 
 static PyObject *Proxy_repr(ProxyObject *self)
 {
-    Proxy__ENSURE_WRAPPED_OR_RETURN_NULL(self);
 
-#if PY_MAJOR_VERSION >= 3
-    return PyUnicode_FromFormat("<%s at %p for %s at %p>",
-            Py_TYPE(self)->tp_name, self,
-            Py_TYPE(self->wrapped)->tp_name, self->wrapped);
-#else
-    return PyString_FromFormat("<%s at %p for %s at %p>",
-            Py_TYPE(self)->tp_name, self,
-            Py_TYPE(self->wrapped)->tp_name, self->wrapped);
+#if PY_MAJOR_VERSION < 3
+    PyObject *factory_repr;
+
+    factory_repr = PyObject_Repr(self->factory);
+    if (factory_repr == NULL)
+        return NULL;
 #endif
+
+    if (self->wrapped) {
+#if PY_MAJOR_VERSION >= 3
+        return PyUnicode_FromFormat("<%s at %p wrapping %R at %p with factory %R>",
+                Py_TYPE(self)->tp_name, self,
+                self->wrapped, self->wrapped,
+                self->factory);
+#else
+        PyObject *wrapped_repr;
+
+        wrapped_repr = PyObject_Repr(self->wrapped);
+        if (wrapped_repr == NULL)
+            return NULL;
+
+        return PyString_FromFormat("<%s at %p wrapping %s at %p with factory %s>",
+                Py_TYPE(self)->tp_name, self,
+                PyString_AS_STRING(wrapped_repr), self->wrapped,
+                PyString_AS_STRING(factory_repr));
+#endif
+    } else {
+#if PY_MAJOR_VERSION >= 3
+        return PyUnicode_FromFormat("<%s at %p with factory %R>",
+                Py_TYPE(self)->tp_name, self,
+                self->factory);
+#else
+        return PyString_FromFormat("<%s at %p with factory %s>",
+                Py_TYPE(self)->tp_name, self,
+                PyString_AS_STRING(factory_repr));
+#endif
+    }
 }
 
 /* ------------------------------------------------------------------------- */
@@ -844,6 +888,15 @@ static PyObject *Proxy_reversed(
             self->wrapped, NULL);
 }
 
+/* ------------------------------------------------------------------------- */
+static PyObject *Proxy_reduce(
+        ProxyObject *self, PyObject *args)
+{
+    Proxy__ENSURE_WRAPPED_OR_RETURN_NULL(self);
+
+    return Py_BuildValue("(O(O))", identity_ref, self->wrapped);
+}
+
 /* ------------------------------------------------------------------------- */
 
 #if PY_MAJOR_VERSION >= 3
@@ -1014,10 +1067,8 @@ static PyObject *Proxy_get_wrapped(
 static int Proxy_set_wrapped(ProxyObject *self,
         PyObject *value)
 {
-    Proxy__ENSURE_WRAPPED_OR_RETURN_MINUS1(self);
-
     if (value) Py_INCREF(value);
-    Py_DECREF(self->wrapped);
+    Py_XDECREF(self->wrapped);
 
     self->wrapped = value;
 
@@ -1026,6 +1077,28 @@ static int Proxy_set_wrapped(ProxyObject *self,
 
 /* ------------------------------------------------------------------------- */
 
+static PyObject *Proxy_get_factory(
+        ProxyObject *self)
+{
+    Py_INCREF(self->factory);
+    return self->factory;
+}
+
+/* ------------------------------------------------------------------------- */
+
+static int Proxy_set_factory(ProxyObject *self,
+        PyObject *value)
+{
+    if (value) Py_INCREF(value);
+    Py_XDECREF(self->factory);
+
+    self->factory = value;
+
+    return 0;
+}
+
+/* ------------------------------------------------------------------------- */
+
 static PyObject *Proxy_getattro(
         ProxyObject *self, PyObject *name)
 {
@@ -1086,31 +1159,6 @@ static PyObject *Proxy_getattr(
 static int Proxy_setattro(
         ProxyObject *self, PyObject *name, PyObject *value)
 {
-    static PyObject *self_str = NULL;
-    static PyObject *wrapped_str = NULL;
-    static PyObject *startswith_str = NULL;
-
-    PyObject *match = NULL;
-
-    if (!startswith_str) {
-#if PY_MAJOR_VERSION >= 3
-        startswith_str = PyUnicode_InternFromString("startswith");
-#else
-        startswith_str = PyString_InternFromString("startswith");
-#endif
-    }
-
-    if (!wrapped_str) {
-#if PY_MAJOR_VERSION >= 3
-        wrapped_str = PyUnicode_InternFromString("__wrapped__");
-#else
-        wrapped_str = PyString_InternFromString("__wrapped__");
-#endif
-    }
-
-    if (PyObject_RichCompareBool(name, wrapped_str, Py_EQ) == 1)
-        return PyObject_GenericSetAttr((PyObject *)self, name, value);
-
     if (PyObject_HasAttr((PyObject *)Py_TYPE(self), name))
         return PyObject_GenericSetAttr((PyObject *)self, name, value);
 
@@ -1232,6 +1280,8 @@ static PyMethodDef Proxy_methods[] = {
                     METH_VARARGS , 0 },
     { "__bytes__",  (PyCFunction)Proxy_bytes, METH_NOARGS, 0 },
     { "__reversed__", (PyCFunction)Proxy_reversed, METH_NOARGS, 0 },
+    { "__reduce__", (PyCFunction)Proxy_reduce, METH_NOARGS, 0 },
+    { "__reduce_ex__", (PyCFunction)Proxy_reduce, METH_O, 0 },
 #if PY_MAJOR_VERSION >= 3
     { "__round__",  (PyCFunction)Proxy_round, METH_NOARGS, 0 },
 #endif
@@ -1253,6 +1303,8 @@ static PyGetSetDef Proxy_getset[] = {
                             (setter)Proxy_set_annotations, 0 },
     { "__wrapped__",        (getter)Proxy_get_wrapped,
                             (setter)Proxy_set_wrapped, 0 },
+    { "__factory__",        (getter)Proxy_get_factory,
+                            (setter)Proxy_set_factory, 0 },
     { NULL },
 };
 
@@ -1311,14 +1363,14 @@ PyTypeObject Proxy_Type = {
 #if PY_MAJOR_VERSION >= 3
 static struct PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT,
-    "cext",              /* m_name */
-    NULL,                /* m_doc */
-    -1,                  /* m_size */
-    NULL,                /* m_methods */
-    NULL,                /* m_reload */
-    NULL,                /* m_traverse */
-    NULL,                /* m_clear */
-    NULL,                /* m_free */
+    "lazy_object_proxy.cext", /* m_name */
+    NULL,                     /* m_doc */
+    -1,                       /* m_size */
+    module_functions,         /* m_methods */
+    NULL,                     /* m_reload */
+    NULL,                     /* m_traverse */
+    NULL,                     /* m_clear */
+    NULL,                     /* m_free */
 };
 #endif
 
@@ -1326,11 +1378,12 @@ static PyObject *
 moduleinit(void)
 {
     PyObject *module;
+    PyObject *dict;
 
 #if PY_MAJOR_VERSION >= 3
     module = PyModule_Create(&moduledef);
 #else
-    module = Py_InitModule3("cext", NULL, NULL);
+    module = Py_InitModule3("lazy_object_proxy.cext", module_functions, NULL);
 #endif
 
     if (module == NULL)
@@ -1339,8 +1392,13 @@ moduleinit(void)
     if (PyType_Ready(&Proxy_Type) < 0)
         return NULL;
 
-    if (PyType_Ready(&Proxy_Type) < 0)
+    dict = PyModule_GetDict(module);
+    if (dict == NULL)
         return NULL;
+    identity_ref = PyDict_GetItemString(dict, "identity");
+    if (identity_ref == NULL)
+        return NULL;
+    Py_INCREF(identity_ref);
 
     Py_INCREF(&Proxy_Type);
     PyModule_AddObject(module, "Proxy",
